%Tutorial 2: The Fourier Transform

%This is a tutorial on a particularly well-studied set of basis functions 
%that give rise to the Fourier series and the Fourier transform. Emphasis 
%is placed on the implicit role of periodicity in applying the Fourier
%transform to finite time-series data. 

%% BEGIN SECTION 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%   SECTION 1: The period functional Per(f) and how it varies for Per(f+g).
    clear;
    
    %Period is the (non-negative) inverse of frequency. Therefore, if 
    %f = sin(wt), the Per(f) = abs(1/w).
    %
    %Suppose we had a series of periodic function [f1 ... fN] with
    %frequencies [1 ... N]. Then 
    %
    %   [Per(f1) ... Per(fN)] = [abs(1/1) ... abs(1/N)].
    %
    %If f and g are periodic functions, it is almost always true that: 
    %   
    %   Per(f + g) = lcm(Per(f),Per(g)).
    %
    %where lcm(m,n) is the least common multiple function extended to the 
    %rationals.
    
    %N      :   The number of periodic function.
    %w      :   The frequency of each function in [1 N].
    %t      :   A variable representing time.
    %F      :   F(w,:) = fw = sin(w*t).
    
    %Constructing F(w,t).
    N = 15;
    t = linspace(0,2*pi,1000);
    F = zeros(N,length(t));
    for w = 1:N
        F(w,:) = sin(w*t);
    end
    
    %Constructing PerF = c*Per(F(w,:)), where c = factorial(N). Values in
    %PerF are natural numbers so as to be compatible with lcm. The true 
    %period of F(w,:) = PerF(w)/c.
    %
    %PerFF(i,j) = Per(F(i,:) + F(j,:)) = lcm(Per(F(i,:)),Per(F(j,:))).
    
    w = [1:N];
    c = factorial(N);
    PerF = zeros(1,N);
    for k = 1:N
        PerF(k) = prod(setdiff(w,k));
    end
    PerFF = zeros(N,N);
    for i = 1:N
        for j = 1:i
            PerFF(i,j) = lcm(PerF(i),PerF(j))/c;
            PerFF(j,i) = PerFF(i,j);
        end
    end
    
    %Visualizing F and PerFF.
    figure;
    setP([393 502 1526 502]);
    subplot(1,2,1);
        imagesc(t,w,F); h = colorbar;
        title([int2str(N) ' sinusoids with integral-valued frequencies']);
        setXTL; setYT(w); setYTL(w);
        xlabel('Time');
        ylabel('Frequency w');
        cblabel(h,'Height');
    subplot(1,2,2);
        imagesc(w,w,PerFF); h = colorbar;
        title('The periods of sums of sinusoids: PerFF(w1,w2)');
        xlabel('Frequency = w1');
        ylabel('Frequency = w2');
        cblabel(h,'Period');

    % QUESTIONS:
    %
    % 1) Open a new script called garbage, by typing 'edit garbage' in the
    %    command line. Suppose you had a sequence of periodic functions G
    %    with periods (not frequencies) PerG = [1 ... 50]. Compute PerGG 
    %    and display it using the imagesc function. Comment on the 
    %    aesthetic nature of the lcm function.    
    %
    % 2) Let eps > 0 be a small number that you imagine to take on any 
    %    value close to zero. For an arbitrary w, as eps => 0 is 
    %       
    %       P(w,eps) = Per(sin(wt) + sin((w + eps)t))
    %
    %    well-behaved? Why or why not?
    %
    %    Hint: Consider all natural numbers k, such that eps = w/k.
    %    Construct the behavior of more sequences of w by considering the 
    %    initial frequency to change from w to 1/P(w,eps), for any 
    %    eps = w/k.

    
%% END SECTION 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%% COMMENT OUT SECTION 1. UNCOMMENT SECTION 2 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


% %% BEGIN SECTION 2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % 
% %   SECTION 2: The complex exponential basis functions
%     clear;
%     
%     %The main idea behind the Fourier transform is that sines and cosines
%     %of specific frequencies are a set of orthonormal basis functions. The
%     %goal is to examine the complex exponential, and its relationship to 
%     %sines and cosines via Euler's identity:
%     %
%     %   a*exp(i*w*t + theta) = a*cos(w*t + theta) + i*a*sin(w*t + theta);
%     
%     % N         :   N is the number of data points in the time series.
%     % t         :   A time variable.
%     % a         :   An amplitude.
%     % w         :   A frequency.
%     % theta     :   An initial phase.
%     % f         :   A complex exponential.
%     % F         :   An [N N] orthonormal basis.
%     
%     %initializing a complex exponential.
%     N = 200;
%     a = 1;
%     w = 1;
%     theta = 0;
%     t = linspace(0,2*pi,N);
%     f = a*exp(2*pi*sqrt(-1)*w*t + theta);
%     
%     %Plotting the complex exponential in 2 and 3 dimensions.
%     h = figure;
%     setP([391 109 1177 807]);
%     subplot(1,2,1);
%         plot(real(f),imag(f),'Color',[0 0 .8],'LineWidth',2);
%         axis([-a-a/10 a+a/10 -a-a/10 a+a/10]);
%         axis square; setXT([-a 0 a]); setYT([-a 0 a]);
%         title('The Complex Exponential');
%         xlabel('real(f)');
%         ylabel('imag(f)');
%     subplot(1,2,2)
%         plot3(t,real(f),imag(f),'Color',[0 0 .8],'LineWidth',2);
%         axis([0 2*pi -a-a/10 a+a/10 -a-a/10 a+a/10]);
%         setXT; setYT([-a 0 a]); setZT([-a 0 a]);
%         title('The Complex Exponential Helix');
%         xlabel('time');
%         ylabel('real(f)');
%         zlabel('imag(f)');
%         
%     %initializing the Fourier basis.
%     N = 50;
%     i = sqrt(-1);
%     w = [0:N-1]./N;
%     t = [0:N-1];
%     F = zeros(N,N);
%     for k = 1:N;
%         F(:,k) = sqrt((1/N))*exp(2*pi*i*w(k)*t);
%     end
%     
%     figure; 
%     setP([394 46 1175 956]);
%     subplot(2,2,1);
%         imagesc(real(F));
%         colorbar;
%         title('Fourier Basis:  real(F)');
%         xlabel('Frequency');
%         ylabel('Time');
%     subplot(2,2,3);
%         imagesc(imag(F));
%         colorbar;
%         title('Fourier Basis:  imag(F)');
%         xlabel('Frequency');
%         ylabel('Time');
%     subplot(2,2,2);
%         imagesc(angle(F)); 
%         caxis([-pi pi]);
%         colorbar;
%         title('Fourier Basis:  angle(F)');
%         xlabel('Frequency');
%         ylabel('Time');
%     subplot(2,2,4);
%         imagesc(real(F'*F)); colorbar;
%         title('Orthogonality:  F''*F');
%         xlabel('Frequency');
%         ylabel('Frequency');
%     figure(h);
%     
%     
%     % QUESTIONS:
%     %
%     % 1) For a time-series X = [x1 ... xN], the Discrete Fourier Transform 
%     %    (DFT) of W = F'X'. Every orthonormal basis can be thought of as a
%     %    rotation of some set of data. In this case, how many data points
%     %    are there, and what is the dimensionality of the data with respect 
%     %    to rotation?
%     %
%     %    Hint: Remember that F is a complex basis. Factor this in to
%     %    considerations of dimensionality.
%     %
%     % 2) For X = [x1 ... xN], N/2 is considered to be the Nyquist
%     %    frequency, the largest frequency that can be represented in a
%     %    time-series of length N. Yet there would appear to be N/2 higher
%     %    frequencies. Why is this not the case?
%     %
%     %    Let S be a [N N] matrix, with:
%     %
%     %       N = 10;
%     %       w = [0:N-1]./N;
%     %       t = [0:N-1];
%     %       i = sqrt(-1);
%     %       S(k,n) = sin(2*pi*i*w(k)*t(n));
%     %
%     %    Use imagesc to plot S, and compare the sinusoids with w(k) < N/2
%     %    to those with w(k) > N/2. This effect is called aliasing. Describe
%     %    the intuition behind aliasing.
%     %
%     % 3) Choose natural numbers k1 and k2, and construct three time-series: 
%     %       
%     %        s1 = sin(2*pi*i*w(k1)*t) + sin(2*pi*i*w(k2)*t);
%     %        s2 = cos(2*pi*i*w(k1)*t) + cos(2*pi*i*w(k2)*t);
%     %        s3 = sin(2*pi*i*(w(k1)+.1)*t) + sin(2*pi*i*(w(k2)+.1)*t);
%     %
%     %    Take the Fourier Transform of each using the fft algorithm:
%     %
%     %       Sk = fft(sk);
%     %
%     %   For each, plot the amplitude of Sk = abs(Sk), and the phase of 
%     %   Sk = angle(Sk);
%     %
%     %   Describe the differences in each transformed series with respect to 
%     %   amplitude and phase.
%     %
%     % 4) For vectors X = [x1 ... xN] and Y = [y1 ... yn], each with zero 
%     %    mean, the covariance between X and Y, cov(X,Y) = X*Y', is simply
%     %    the dot product between the two. 
%     %
%     %    For a basis B, consider the columns to be zero mean 'shapes'. In 
%     %    the Fourier basis, the shapes are helices. What can be said about 
%     %    the different shapes of a basis with respect to covariance? 
%     %
%     %    By a similar analogy, projecting data onto a single dimension is 
%     %    akin to comparing it to a shape. With respect to covariance, what 
%     %    is the relationship between the value of the transformed data 
%     %    along a single dimension and the shape that represents that
%     %    dimension?
% 
%     
% %% END SECTION 2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%