%Tutorial 3: The Hilbert Transform

%The Hilbert transform was introduced by David Hilbert around 1900, and
%grew out of Hilbert's work on mapping periodic functions onto the unit
%circle. The transform appeared first in a joint proof between Hilbert and 
%Euler concerning the nature of holomorphic functions in the upper half 
%plane of the complex sphere. Despite its arcane beginnings, it has
%ubiquitious use in modern society; it is the substrate of amplitude
%modulation coding in radio transmission. Ultimately, what the Hilbert
%transform provides is a way of defining the instantaneous amplitude and
%instantaneous phase of an oscillating, band-limited signal. These are 
%invaluable for interpreting approximately continuous-valued data, and is a
%personal favorite of mine in the signal processing toolbox.

%% BEGIN SECTION 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%   SECTION 1: The Hilbert Transform and the analytic signal.
    clear;
    
    %The Hilbert Transform H{f(t)} can be defined several ways:
    %
    %   h(t) = 1/(pi*t),
    %   H(f) = conv(h,f).
    %
    %While this is simple and linearly oriented, it gives little intuition 
    %as the nature of the transform itself. A second way makes reference to
    %the Fourier transform. Using it, we can decompose band-limited 
    %continuous signal and all discrete, finite time-series. We can define
    %the Hilbert transform with respect to this decomposition.
    %
    %   There exist [a1 ... aN], [o1 ... oN] such that
    %
    %       f(t) = a1*sin(2*pi*1*t + o1)  +  ...  +  aN*sin(2*pi*N*t + oN).
    %
    %   H(f) can be defined by replacing the sines by cosines:
    %
    %       H(f) = a1*cos(2*pi*1*t + o1)  +  ...  +  aN*cos(2*pi*N*t + oN).
    %
    %Recall that cos(t) = sin(t - pi/2). Therefore the Hilbert Transform phase-shifts f by pi/2 across all
    %frequencies.
    %
    %The derivative operator, denoted D, applied to a sine wave, acts in 
    %the following way: D(sin(t)) = cos(t) = sin(t - pi/2). The successive 
    %derivatives of sine are rotations by pi/2 which have the following 
    %symmetries:
    %
    %   DD(sin)     =  -sin
    %   DDDD(sin)   =   sin
    %
    %The Hilbert transform extends this notion of rotation to an arbitrary
    %band-limited function f, such that:
    %
    %   HH(f) = -f
    %   HHHH(f) = f
    %
    %In this sense, the Hilbert Transform allows you to interpret any
    %function as a sine wave.
    %
    %F, the analytic function of a signal f is: 
    %
    %   F = f + i*H(f).
    %   f  = real(F).
    %
    %In matlab, hilbert(f) returns the analytic function of f. Therefore,
    %
    %   H(f) = imag(hilbert(f)).
    %
    %Here's a useful decomposition based on the Hilbert transform with a
    %simple proof. Let:
    %         
    %         Hf    =   imag(hilbert(f))        which is H(f)
    %         a(t)  =   sqrt(f.^2+  Hf.^2)      an amplitude function
    %       phi(t)  =   atan2(Hf,f)             a phase function
    %
    %   f(t)   =    real(f(t) + i*H(f(t)))
    %          =    real(a(t)*exp(i*phi(t)))
    %          =    real(a(t)*cos(phi(t)) + i*a(t)*sin(phi(t)))
    %          =    a(t)*cos(phi(t)).
    %
    %Therefore, the Hilbert Transform allows us to factorize f into
    %instantaneous amplitude and phase. Very useful.
    
    %Let's understand the Hilbert transform with respect to a dampened
    %chirp.
    %   N   :   The length of the signal.
    %   t   :   The time-course of the signal.
    %   A   :   The time-varying amplitude of the signal.
    %   W   :   The time-varying frequency of the signal.
    %   f   :   The dampened chirp.
    %   F   :   The 
    
    N = 5000;
    C = 50;
    t = linspace(-10*pi,(C+10)*pi,N);
    fnd = find(t >= 0 & t <= C*pi);
    A = exp(-.01*t);
    W = exp(-.02*t) + 1;
    f = A.*sin(W.*t);
    F = hilbert(f);
    [a s phi] = hilfactor(f);
    
    F = F(fnd);
    a = a(fnd);
    s = s(fnd);
    phi = phi(fnd);
    f = f(fnd);
    t = t(fnd);
    
    %Visualization using amplitude / phase analysis.
    h = figure;
    setP([2 582 1916 502]);
    subplot(1,2,1);
        plot(t,f,'k','lineWidth',3); hold on;
        plot(t,a,'r',t,s,'b'); axis tight;
        title('The Hilbert Factorization of a dampened chirp');
        xlabel('Time'); setXTL;
        ylabel('Signal'); setYTL;
    subplot(1,2,2);
        plot3(t,real(F),imag(F),'k','LineWidth',2);
        xlabel('Time'); setXTL;
        ylabel('real(f)'); setYTL;
        zlabel('imag(f)'); setZTL;
        axis tight;
        
    figure;
    setP([5 49 1913 531]);
    plot(real(F),imag(F),'k','LineWidth',2);
    xlabel('real(f)'); setXTL;
    ylabel('imag(f)'); setYTL;
    axis square;
    figure(h);
    
    %QUESTIONS:
    %
    % 1) What is H(sin(wt + phi)). Plot the real and imaginary parts of the
    %    analytic signal against each other and see what you get. What is
    %    the relation.
    %
    % 2) Now ass a few sines of different frequencies and phases. Visualize
    %    the function along with the amplitude and cos(phase) using 
    %    hilfactor. What is the Hilbert factorization doing?
    
%% END SECTION 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%