% Tutorial 2: Basics of Information theory

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SECTION 1: Information content in arbitrary distributions

% Before going to spike trains, Let us spend some time getting a feel for
% the information content of some arbitrary distributions. For any discrete
% distribution p(xi) for i = [1, ... N], with the following conditions: 
%
%    p(xi) > 0
%    sum(p(xi)) = 1,
%
%the entropy of that distribution is given by 
%                   
%    H(p) = -sum(p(xi)*log2(p(xi)));
%
% It's worth noting that entropy is permutation invariant. If f is a 
% permutation of [1 ... N], 
% 
%    H(p(xi)) = H(p(xf(i)),
%
% where i and f(i) denote subscripts. the entropy. This condition can be
% made stronger. It holds for all f(i) that are invertible. 

setPause = 0;
clc; 

display('Creating a probability distribution with N points where:')
N = 20; % Number of xi's in the distribution
p = rand(N,1); % Our probability distribution of choice
N

display('The probability distribution is stored in ''p''');
if setPause display('Press a key to continue...'); pause(); end

display('Is our probability distribution valid?');
display('Checking if all p(xi)>0. isempty(find(p<0))?');
isempty(find(p<0))
if setPause display('Press a key to continue...'); pause(); end

display('Now, checking if p(xi) sum to 1? sum(p(:))');
sum(p(:))
if setPause display('Press a key to continue...'); pause(); end

display('Making p into a valid PDF');
p = hist(p,unique(p));
p = p./sum(p);
if setPause display('Press a key to continue...'); pause(); end

% Plotting
display('Plotting the distribution');
figure;
% plotyy(1:length(p),p,1:length(p),pCumSum,'bar','line')
bar(p,'FaceColor','k');
pCumSum = cumsum(p);
hold on
plot(pCumSum,'g--');
hold off
set(gca,'xlim',[1 length(p)]); set(gca,'ylim',[0 1]);
title('The PDF and its CDF'); legend('PDF','CDF');
if setPause display('Press a key to continue...'); pause(); end

% Use entropy function to calculate the entropy of the current distribution
display('');display('');display('entropy(p):')
S = entropy(p)
if setPause display('Press a key to continue...'); pause(); end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EXERCISE 1: Find that distribution of size N that gives you maximum     %
% entropy. What is that maximum?                                          %
% EXERCISE 2: Prove that for all f|f is invertible, S(f(x)) = S(x) where S%
% is the entropy function                                                 %
% EXERCISE 3: Prove that if f is not invertible, S(f(x))<S(x)             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SECTION 2: Calculating Mutual information between two variables

% Let x and y be two variables sampled from the joint probability
% distribution p(x,y)
setPause = 1;
clc;

display('Create a random matrix');
p = rand(10,10);
display(p);
display(' ');
display('sum(p) : ');
display(sum(p(:)));
display(' ');
display('''p'' is not a PDF')
indXs = 1:10;
indYs = 1:10;
if setPause display('Press a key to continue...'); pause(); end
display('---------------------------------------- ');

display('Make p a PDF');;
p = p./gsum(p);
display(p);
display(' ');
display('sum(p) : ');
display(sum(p(:)));
if setPause display('Press a key to continue...'); pause(); end
display('---------------------------------------- ');

display('How many samples (N) do we need to collect to correctly estimate the probability distribution function?')
if setPause display('Press a key to continue...'); pause(); end
display('Here are some guidelines...');
display('   1. We need to sample each bin atleast once to get a semblance of a PDF');
display('           Therefore, N > sizeX(p)*sizeY(p)');
display('   2. We might need to sample each bin atleast a few times(n) to ensure that noise does not overwhelm our estimate');
display('           lets take n=10. Therefore N > n*sizeX(p)*sizeY(p)');
display('   3. The likelihood that the lowest probability bin has n samples is inversely proportional to max(p)/min(p)');
display('           so lets oversample by that factor. N > n*sizeX(p)*sizeY(p)*[max(p)/min(p)]');

n = 10;
N = round(n*length(p(:))*(max(p(:))/min(p(:))))
if setPause display('Press a key to continue...'); pause(); end
display('---------------------------------------- ');

resetN = input('Do you want to reset N? (y/n) :','s');
if strcmp(resetN,'y')
    N = input('Enter the new N : ');
    display(' ');
end

display('Lets sample from the distribution using the sample2D function');
[Xs Ys] = sample2D(p,N);
if setPause display('Press a key to continue...'); pause(); end
display('---------------------------------------- ');

display('Now lets Actually calculate the Mutual Information available between these two variables');
display('We will use the mutualinfo function');
I = mutualinfo(Xs,Ys,indXs,indYs)
if setPause display('Press a key to continue...'); pause(); end
display('=========================================================================================');

display('Just for kicks, lets take a PDF where the Xs and Ys are independent');
display('one example of such a function is the constant probability function');
p = dataToPDF(ones(10,10));
display(p);
display(' ');
display('sum(p) : ');
display(sum(p(:)));
display(' ');
N = round(n*length(p(:))*(max(p(:))/min(p(:))))
display(' ');
display('Lets sample from the distribution using the sample2D function....');
[Xs Ys] = sample2D(p,N);
display(' ');
display('...and calculate the Mutual Information available between these two variables');
I = mutualinfo(Xs,Ys,indXs,indYs)
display(' ');
display('Is I=0? If so why? If not why not?');
display(' ');
if setPause display('Press a key to continue...'); pause(); end
display('=========================================================================================');

display('Lets reset N to ten times its original value and recompute');
N = round(n*length(p(:))*(max(p(:))/min(p(:))))*10
[Xs Ys] = sample2D(p,N);
I = mutualinfo(Xs,Ys,indXs,indYs)
display(' ');
display('What does this tell you?');
display(' ');
if setPause display('Press a key to continue...'); pause(); end
display('=========================================================================================');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EXERCISE 1: If f is invertible, I(f(x),y) = I(x,y) where I is the Mutual%
% information function. True/False                                        %
% EXERCISE 2: I(f(x),y) < I(x,y) if f is not invertible. True/False       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
